module AmazonS3IndexProxy
  class Proxy
    attr_reader :index_file_url, :options

    def initialize(index_file_url, cache_options={})
      @index_file_url = index_file_url
      @options = { expires_in: expires_in }.merge(cache_options)
    end

    def call
      index_file
    end

    private

    def expires_in
      AmazonS3IndexProxy.expires_in
    end

    def index_file
      Rails.cache.fetch(cache_key, options) do
        Rails.logger.info("Updating cached version of '#{index_file_url}'")
        fetch_index_file
      end
    end

    def cache_key
      [
        AmazonS3IndexProxy.cache_key_prefix,
        AmazonS3IndexProxy.index_file_url,
      ].join('\\')
    end

    def fetch_index_file
      Net::HTTP.get(index_file_uri)
    end

    def index_file_uri
      URI(index_file_url)
    end
  end
end
